Index: main/db.c
===================================================================
--- main/db.c	(revision 419309)
+++ main/db.c	(working copy)
@@ -119,7 +119,8 @@
 #define DEFINE_SQL_STATEMENT(stmt,sql) static sqlite3_stmt *stmt; \
 	const char stmt##_sql[] = sql;
 
-DEFINE_SQL_STATEMENT(put_stmt, "INSERT OR IGNORE INTO astdb (key, value) VALUES (?1, ?2); UPDATE astdb SET value=?2 WHERE changes()=0 AND key=?1")
+DEFINE_SQL_STATEMENT(put_stmt, "INSERT OR IGNORE INTO astdb (key, value) VALUES (?, ?)")
+DEFINE_SQL_STATEMENT(update_stmt, "UPDATE astdb SET value=? WHERE changes()=0 AND key=?")
 DEFINE_SQL_STATEMENT(get_stmt, "SELECT value FROM astdb WHERE key=?")
 DEFINE_SQL_STATEMENT(del_stmt, "DELETE FROM astdb WHERE key=?")
 DEFINE_SQL_STATEMENT(deltree_stmt, "DELETE FROM astdb WHERE key || '/' LIKE ? || '/' || '%'")
@@ -171,6 +172,7 @@
 	clean_stmt(&gettree_all_stmt, gettree_all_stmt_sql);
 	clean_stmt(&showkey_stmt, showkey_stmt_sql);
 	clean_stmt(&put_stmt, put_stmt_sql);
+	clean_stmt(&update_stmt, update_stmt_sql);
 	clean_stmt(&create_astdb_stmt, create_astdb_stmt_sql);
 }
 
@@ -185,6 +187,7 @@
 	|| init_stmt(&gettree_stmt, gettree_stmt_sql, sizeof(gettree_stmt_sql))
 	|| init_stmt(&gettree_all_stmt, gettree_all_stmt_sql, sizeof(gettree_all_stmt_sql))
 	|| init_stmt(&showkey_stmt, showkey_stmt_sql, sizeof(showkey_stmt_sql))
+	|| init_stmt(&update_stmt, update_stmt_sql, sizeof(update_stmt_sql))
 	|| init_stmt(&put_stmt, put_stmt_sql, sizeof(put_stmt_sql));
 }
 
@@ -257,7 +260,6 @@
 		ast_mutex_unlock(&dblock);
 		return -1;
 	}
-
 	ast_mutex_unlock(&dblock);
 
 	return 0;
@@ -332,8 +334,20 @@
 		ast_log(LOG_WARNING, "Couldn't execute statment: %s\n", sqlite3_errmsg(astdb));
 		res = -1;
 	}
+	sqlite3_reset(put_stmt);
 
-	sqlite3_reset(put_stmt);
+	if (sqlite3_bind_text(update_stmt, 2, fullkey, fullkey_len, SQLITE_STATIC) != SQLITE_OK) {
+		ast_log(LOG_WARNING, "Couldn't bind key to stmt: %s\n", sqlite3_errmsg(astdb));
+		res = -1;
+	} else if (sqlite3_bind_text(update_stmt, 1, value, -1, SQLITE_STATIC) != SQLITE_OK) {
+		ast_log(LOG_WARNING, "Couldn't bind value to stmt: %s\n", sqlite3_errmsg(astdb));
+		res = -1;
+	} else if (sqlite3_step(update_stmt) != SQLITE_DONE) {
+		ast_log(LOG_WARNING, "Couldn't execute statment: %s\n", sqlite3_errmsg(astdb));
+		res = -1;
+	}
+	sqlite3_reset(update_stmt);
+
 	db_sync();
 	ast_mutex_unlock(&dblock);
 
